@@ -29,6 +29,7 @@
 # YR 09 Feb 2016: first version
 # YR 19 Mar 2017: 10)
 # YR 03 Okt 2018: a few functions gain a wt= argument
+# YR 01 Jul 2018: first_order functions gain cluster.idx= argument
 
 
 # 1) likelihood
@@ -36,7 +37,7 @@
 # 1a: input is raw data
 #  - two strategies: 1) using missing patterns (pattern = TRUE)
 #                    2) truly case per case    (pattern = FALSE)
-#    depending on the sample size, missing patterns, etc... one can be 
+#    depending on the sample size, missing patterns, etc... one can be
 #    (much) faster than the other
 lav_mvnorm_missing_loglik_data <- function(Y           = NULL,
                                            Mu          = NULL,
@@ -49,17 +50,13 @@ lav_mvnorm_missing_loglik_data <- function(Y           = NULL,
                                            log2pi      = TRUE,
                                            minus.two   = FALSE) {
 
-    if(!is.null(x.idx) && length(x.idx) > 0L) {
-        warning("lavaan WARNING: x.idx not supported yet (ignored)")
-    }
-
     if(pattern) {
-        llik <- lav_mvnorm_missing_llik_pattern(Y = Y, wt = wt, Mu = Mu, 
-                    Sigma = Sigma, Sinv.method = Sinv.method,
+        llik <- lav_mvnorm_missing_llik_pattern(Y = Y, wt = wt, Mu = Mu,
+                    Sigma = Sigma, x.idx = x.idx, Sinv.method = Sinv.method,
                     log2pi = log2pi, minus.two = minus.two)
     } else {
-        llik <- lav_mvnorm_missing_llik_casewise(Y = Y, wt = wt, Mu = Mu, 
-                    Sigma = Sigma, Sinv.method = Sinv.method,
+        llik <- lav_mvnorm_missing_llik_casewise(Y = Y, wt = wt, Mu = Mu,
+                    Sigma = Sigma, x.idx = x.idx, Sinv.method = Sinv.method,
                     log2pi = log2pi, minus.two = minus.two)
     }
 
@@ -68,7 +65,7 @@ lav_mvnorm_missing_loglik_data <- function(Y           = NULL,
     } else {
         loglik <- sum(llik, na.rm = TRUE)
     }
-    
+
     loglik
 }
 
@@ -83,10 +80,9 @@ lav_mvnorm_missing_loglik_samplestats <- function(Yp          = NULL,
                                                   log2pi      = TRUE,
                                                   minus.two   = FALSE) {
 
-    if(!is.null(x.idx) && length(x.idx) > 0L) {
-        #warning("lavaan WARNING: x.idx not supported yet (ignored)")
-        
-    }
+    #if(!is.null(x.idx) && length(x.idx) > 0L) {
+    #    #warning("lavaan WARNING: x.idx not supported yet (ignored)")
+    #}
 
     LOG.2PI <- log(2*pi); pat.N <- length(Yp);  P <- length(Yp[[1]]$var.idx)
 
@@ -137,18 +133,26 @@ lav_mvnorm_missing_loglik_samplestats <- function(Yp          = NULL,
     }
 
     # x.idx
-    if(!is.null(x.idx) && length(x.idx) > 0L) {
-        #warning("lavaan WARNING: x.idx not supported yet (ignored)")
+    if(length(x.idx) > 0L) {
+        stopifnot(!is.null(x.cov), !is.null(x.mean))
         P.x <- length(x.idx); N <- sum(sapply(Yp, "[[", "freq"))
         Sigma.x <- Sigma[x.idx, x.idx, drop = FALSE]
         Sigma.inv.x <- lav_matrix_symmetric_inverse(S = Sigma.x, logdet = TRUE,
                                               Sinv.method = Sinv.method)
         logdet.x <- attr(Sigma.inv.x, "logdet")
 
+        # tr(Sigma^{-1} %*% S)
+        DIST1 <- sum(Sigma.inv.x * x.cov)
+
+        # (ybar - mu)^T %*% Sigma.inv %*% (ybar - mu)
+        Diff <- as.numeric(x.mean - Mu[x.idx])
+
+        DIST2 <- sum(as.numeric(crossprod(Diff, Sigma.inv.x)) * Diff)
+
         if(log2pi) {
-            loglik.x <- -N/2 * (P.x * LOG.2PI + logdet.x + P.x)
+            loglik.x <- -N/2 * (P.x * LOG.2PI + logdet.x + DIST1 + DIST2)
         } else {
-            loglik.x <- -N/2 * (logdet.x + P.x)
+            loglik.x <- -N/2 * (logdet.x + DIST1 + DIST2)
         }
 
         if(minus.two) {
@@ -168,11 +172,18 @@ lav_mvnorm_missing_llik_casewise <- function(Y           = NULL,
                                              wt          = NULL,
                                              Mu          = NULL,
                                              Sigma       = NULL,
+                                             x.idx       = NULL,
                                              Sinv.method = "eigen",
                                              log2pi      = TRUE,
                                              minus.two   = FALSE) {
 
-    P <- NCOL(Y); N <- NROW(Y); LOG.2PI <- log(2*pi); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); LOG.2PI <- log(2*pi); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     # global inverse + logdet
     Sigma.inv <- lav_matrix_symmetric_inverse(S = Sigma, logdet = TRUE,
@@ -183,7 +194,7 @@ lav_mvnorm_missing_llik_casewise <- function(Y           = NULL,
     Yc <- t( t(Y) - Mu )
 
     # DIST/logdet per case
-    DIST <- logdet <- P.LOG.2PI <- rep(as.numeric(NA), N)
+    DIST <- logdet <- P.LOG.2PI <- rep(as.numeric(NA), NY)
 
     # missing pattern per case
     OBS <- !is.na(Y); P.i <- rowSums(OBS)
@@ -191,9 +202,9 @@ lav_mvnorm_missing_llik_casewise <- function(Y           = NULL,
     # constant
     P.LOG.2PI <- P.i * LOG.2PI
 
-    # complete cases first (only an advantage if we have mostly complete 
+    # complete cases first (only an advantage if we have mostly complete
     # observations)
-    other.idx <- seq_len(N)
+    other.idx <- seq_len(NY)
     complete.idx <- which(P.i == P)
     if(length(complete.idx) > 0L) {
         other.idx <- other.idx[-complete.idx]
@@ -236,6 +247,17 @@ lav_mvnorm_missing_llik_casewise <- function(Y           = NULL,
         llik <- llik * wt
     }
 
+    # x.idx
+    if(length(x.idx) > 0L) {
+        llik.x <- lav_mvnorm_missing_llik_casewise(
+                      Y = Y[, x.idx, drop = FALSE],
+                      wt = wt, Mu = Mu[x.idx],
+                      Sigma = Sigma[x.idx, x.idx, drop = FALSE],
+                      x.idx = NULL, Sinv.method = Sinv.method,
+                      log2pi = log2pi, minus.two = minus.two)
+        llik <- llik - llik.x
+    }
+
     llik
 }
 
@@ -245,11 +267,18 @@ lav_mvnorm_missing_llik_pattern <- function(Y           = NULL,
                                             wt          = NULL,
                                             Mu          = NULL,
                                             Sigma       = NULL,
+                                            x.idx       = NULL,
                                             Sinv.method = "eigen",
                                             log2pi      = TRUE,
                                             minus.two   = FALSE) {
 
-    P <- NCOL(Y); N <- NROW(Y); LOG.2PI <- log(2*pi); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); LOG.2PI <- log(2*pi); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     # global inverse + logdet
     Sigma.inv <- lav_matrix_symmetric_inverse(S = Sigma, logdet = TRUE,
@@ -260,7 +289,7 @@ lav_mvnorm_missing_llik_pattern <- function(Y           = NULL,
     Yc <- t( t(Y) - Mu )
 
     # DIST/logdet per case
-    DIST <- logdet <- P.LOG.2PI <- rep(as.numeric(NA), N)
+    DIST <- logdet <- P.LOG.2PI <- rep(as.numeric(NA), NY)
 
     # missing patterns
     if(is.null(Mp)) {
@@ -294,13 +323,13 @@ lav_mvnorm_missing_llik_pattern <- function(Y           = NULL,
         }
 
         if(Mp$freq[p] == 1L) {
-            DIST[case.idx] <- sum(sigma.inv * 
+            DIST[case.idx] <- sum(sigma.inv *
                 crossprod(Yc[case.idx, var.idx, drop = FALSE]))
         } else {
             DIST[case.idx] <-
                 rowSums(Yc[case.idx, var.idx, drop = FALSE] %*% sigma.inv *
                         Yc[case.idx, var.idx, drop = FALSE])
-        }  
+        }
     }
 
     # compute casewise loglikelihoods
@@ -320,6 +349,18 @@ lav_mvnorm_missing_llik_pattern <- function(Y           = NULL,
         llik <- llik * wt
     }
 
+    # x.idx -- using casewise (as patterns for Y may not be the same as
+    #                          patterns for Y[,-x.idx])
+    if(length(x.idx) > 0L) {
+        llik.x <- lav_mvnorm_missing_llik_casewise(
+                      Y = Y[, x.idx, drop = FALSE],
+                      wt = wt, Mu = Mu[x.idx],
+                      Sigma = Sigma[x.idx, x.idx, drop = FALSE],
+                      x.idx = NULL, Sinv.method = Sinv.method,
+                      log2pi = log2pi, minus.two = minus.two)
+        llik <- llik - llik.x
+    }
+
     llik
 }
 
@@ -333,19 +374,21 @@ lav_mvnorm_missing_dlogl_dmu <- function(Y           = NULL,
                                          wt          = NULL,
                                          Mu          = NULL,
                                          Sigma       = NULL,
+                                         x.idx       = NULL,
                                          Sigma.inv   = NULL,
                                          Sinv.method = "eigen") {
 
     SC <- lav_mvnorm_missing_scores_mu(Y = Y, wt = wt, Mu = Mu, Sigma = Sigma,
-              Sigma.inv = Sigma.inv, Sinv.method = Sinv.method)
+              x.idx = x.idx, Sigma.inv = Sigma.inv, Sinv.method = Sinv.method)
 
     colSums(SC, na.rm = TRUE)
 }
 
-# 2abis: using samplestats 
+# 2abis: using samplestats
 lav_mvnorm_missing_dlogl_dmu_samplestats <- function(Yp          = NULL,
                                                      Mu          = NULL,
                                                      Sigma       = NULL,
+                                                     x.idx       = NULL,
                                                      Sigma.inv   = NULL,
                                                      Sinv.method = "eigen") {
     pat.N <- length(Yp);  P <- length(Yp[[1]]$var.idx)
@@ -382,6 +425,11 @@ lav_mvnorm_missing_dlogl_dmu_samplestats <- function(Yp          = NULL,
         dmu[var.idx] <- dmu[var.idx] + (dmu.pattern * Yp[[p]]$freq)
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        dmu[x.idx] <- 0
+    }
+
     dmu
 }
 
@@ -390,12 +438,19 @@ lav_mvnorm_missing_dlogl_dmu_samplestats <- function(Yp          = NULL,
 # 2b: derivative logl with respect to Sigma (full matrix, ignoring symmetry)
 lav_mvnorm_missing_dlogl_dSigma <- function(Y           = NULL,
                                             Mp          = NULL,
-                                            wt          = NULL, 
+                                            wt          = NULL,
                                             Mu          = NULL,
                                             Sigma       = NULL,
+                                            x.idx       = NULL,
                                             Sigma.inv   = NULL,
                                             Sinv.method = "eigen") {
-    P <- NCOL(Y); N <- NROW(Y); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     if(is.null(Sigma.inv)) {
         # invert Sigma
@@ -442,7 +497,7 @@ lav_mvnorm_missing_dlogl_dSigma <- function(Y           = NULL,
 
         if(length(case.idx) > 1L) {
             if(!is.null(wt)) {
-                out <- stats::cov.wt(Y[case.idx, var.idx, drop = FALSE], 
+                out <- stats::cov.wt(Y[case.idx, var.idx, drop = FALSE],
                                      wt = wt[Mp$case.idx[[p]]], method = "ML")
                 SY <- out$cov
                 MY <- out$center
@@ -456,13 +511,18 @@ lav_mvnorm_missing_dlogl_dSigma <- function(Y           = NULL,
 
         # dSigma for this pattern
         dSigma.pattern <- matrix(0, P, P)
-        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv - 
+        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv -
                         (sigma.inv %*% W.tilde %*% sigma.inv))
 
         # update dSigma
         dSigma <- dSigma + (dSigma.pattern * FREQ)
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        dSigma[x.idx, x.idx] <- 0
+    }
+
     dSigma
 }
 
@@ -470,6 +530,7 @@ lav_mvnorm_missing_dlogl_dSigma <- function(Y           = NULL,
 lav_mvnorm_missing_dlogl_dSigma_samplestats <- function(Yp          = NULL,
                                                         Mu          = NULL,
                                                         Sigma       = NULL,
+                                                        x.idx       = NULL,
                                                         Sigma.inv   = NULL,
                                                         Sinv.method = "eigen") {
     pat.N <- length(Yp);  P <- length(Yp[[1]]$var.idx)
@@ -504,13 +565,18 @@ lav_mvnorm_missing_dlogl_dSigma_samplestats <- function(Yp          = NULL,
 
         # dSigma for this pattern
         dSigma.pattern <- matrix(0, P, P)
-        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv - 
+        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv -
                         (sigma.inv %*% W.tilde %*% sigma.inv))
 
         # update dSigma
         dSigma <- dSigma + (dSigma.pattern * Yp[[p]]$freq)
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        dSigma[x.idx, x.idx] <- 0
+    }
+
     dSigma
 }
 
@@ -519,20 +585,27 @@ lav_mvnorm_missing_dlogl_dSigma_samplestats <- function(Yp          = NULL,
 lav_mvnorm_missing_dlogl_dvechSigma <- function(Y           = NULL,
                                                 wt          = NULL,
                                                 Mu          = NULL,
+                                                x.idx       = NULL,
                                                 Sigma       = NULL,
                                                 Sigma.inv   = NULL,
                                                 Sinv.method = "eigen") {
 
-    FULL <- lav_mvnorm_missing_dlogl_dSigma(Y = Y, wt = wt, Mu = Mu, 
-            Sigma = Sigma, Sigma.inv = Sigma.inv, Sinv.method = Sinv.method)
-    as.numeric( lav_matrix_duplication_pre( as.matrix(lav_matrix_vec(FULL)) ) )
+    dSigma <- lav_mvnorm_missing_dlogl_dSigma(Y = Y, wt = wt, Mu = Mu,
+            Sigma = Sigma, x.idx = x.idx, Sigma.inv = Sigma.inv,
+            Sinv.method = Sinv.method)
+
+    dvechSigma <- as.numeric( lav_matrix_duplication_pre(
+                                  as.matrix(lav_matrix_vec(dSigma)) ) )
+
+    dvechSigma
 }
 
 # 2cbis: using samplestats
-lav_mvnorm_missing_dlogl_dvechSigma_samplestats <- 
+lav_mvnorm_missing_dlogl_dvechSigma_samplestats <-
     function(Yp          = NULL,
              Mu          = NULL,
              Sigma       = NULL,
+             x.idx       = NULL,
              Sigma.inv   = NULL,
              Sinv.method = "eigen") {
     pat.N <- length(Yp);  P <- length(Yp[[1]]$var.idx)
@@ -567,9 +640,14 @@ lav_mvnorm_missing_dlogl_dvechSigma_samplestats <-
 
         # dSigma for this pattern
         dSigma.pattern <- matrix(0, P, P)
-        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv - 
+        dSigma.pattern[var.idx, var.idx] <- -(1/2) * (sigma.inv -
                         (sigma.inv %*% W.tilde %*% sigma.inv))
 
+        # fixed.x?
+        if(length(x.idx) > 0L) {
+            dSigma.pattern[x.idx, x.idx] <- 0
+        }
+
         # convert to vechSigma
         dvechSigma.pattern <- as.numeric( lav_matrix_duplication_pre(
                                   as.matrix(lav_matrix_vec(dSigma.pattern)) ) )
@@ -592,10 +670,17 @@ lav_mvnorm_missing_scores_mu <- function(Y           = NULL,
                                          Mp          = NULL,
                                          Mu          = NULL,
                                          Sigma       = NULL,
+                                         x.idx       = NULL,
                                          Sigma.inv   = NULL,
                                          Sinv.method = "eigen") {
 
-    P <- NCOL(Y); N <- NROW(Y); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     if(is.null(Sigma.inv)) {
         # invert Sigma
@@ -612,7 +697,7 @@ lav_mvnorm_missing_scores_mu <- function(Y           = NULL,
     Yc <- t( t(Y) - Mu )
 
     # dmu per case
-    dmu <- matrix(as.numeric(NA), N, P)
+    dmu <- matrix(as.numeric(NA), NY, P)
 
     # for each pattern, compute sigma.inv
     for(p in seq_len(Mp$npatterns)) {
@@ -638,10 +723,16 @@ lav_mvnorm_missing_scores_mu <- function(Y           = NULL,
             Yc[case.idx, var.idx, drop = FALSE] %*% sigma.inv
     }
 
+    # weights
     if(!is.null(wt)) {
         dmu <- dmu * wt
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        dmu[, x.idx] <- 0
+    }
+
     dmu
 }
 
@@ -651,17 +742,24 @@ lav_mvnorm_missing_scores_vech_sigma <- function(Y           = NULL,
                                                  Mp          = NULL,
                                                  Mu          = NULL,
                                                  Sigma       = NULL,
+                                                 x.idx       = NULL,
                                                  Sigma.inv   = NULL,
                                                  Sinv.method = "eigen") {
 
-    P <- NCOL(Y); N <- NROW(Y); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     if(is.null(Sigma.inv)) {
         # invert Sigma
         Sigma.inv <- lav_matrix_symmetric_inverse(S = Sigma, logdet = FALSE,
                                                   Sinv.method = Sinv.method)
     }
-    
+
     # for the tcrossprod
     idx1 <- lav_matrix_vech_col_idx(P); idx2 <- lav_matrix_vech_row_idx(P)
 
@@ -677,7 +775,7 @@ lav_mvnorm_missing_scores_vech_sigma <- function(Y           = NULL,
     Yc <- t( t(Y) - Mu )
 
     # SC
-    SC <- matrix(as.numeric(NA), nrow = N, ncol = length(iSigma))
+    SC <- matrix(as.numeric(NA), nrow = NY, ncol = length(iSigma))
 
     # for each pattern
     for(p in seq_len(Mp$npatterns)) {
@@ -716,10 +814,17 @@ lav_mvnorm_missing_scores_vech_sigma <- function(Y           = NULL,
     # adjust for vech
     SC[,lav_matrix_diagh_idx(P)] <- SC[,lav_matrix_diagh_idx(P)] / 2
 
+    # weights
     if(!is.null(wt)) {
         SC <- SC * wt
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(P, el.idx = x.idx)
+        SC[, !not.x] <- 0
+    }
+
     SC
 }
 
@@ -729,17 +834,24 @@ lav_mvnorm_missing_scores_mu_vech_sigma <- function(Y           = NULL,
                                                     wt          = NULL,
                                                     Mu          = NULL,
                                                     Sigma       = NULL,
+                                                    x.idx       = NULL,
                                                     Sigma.inv   = NULL,
                                                     Sinv.method = "eigen") {
 
-    P <- NCOL(Y); N <- NROW(Y); Mu <- as.numeric(Mu)
+    P <- NCOL(Y); Mu <- as.numeric(Mu)
+    if(!is.null(wt)) {
+        N <- sum(wt)
+    } else {
+        N <- NROW(Y)
+    }
+    NY <- NROW(Y)
 
     if(is.null(Sigma.inv)) {
         # invert Sigma
         Sigma.inv <- lav_matrix_symmetric_inverse(S = Sigma, logdet = FALSE,
                                                   Sinv.method = Sinv.method)
     }
-    
+
     # for the tcrossprod
     idx1 <- lav_matrix_vech_col_idx(P); idx2 <- lav_matrix_vech_row_idx(P)
 
@@ -755,10 +867,10 @@ lav_mvnorm_missing_scores_mu_vech_sigma <- function(Y           = NULL,
     Yc <- t( t(Y) - Mu )
 
     # dmu per case
-    dmu <- matrix(as.numeric(NA), N, P)
+    dmu <- matrix(as.numeric(NA), NY, P)
 
     # SC
-    SC <- matrix(as.numeric(NA), nrow = N, ncol = length(iSigma))
+    SC <- matrix(as.numeric(NA), nrow = NY, ncol = length(iSigma))
 
     # for each pattern, compute Yc %*% sigma.inv
     for(p in seq_len(Mp$npatterns)) {
@@ -803,10 +915,17 @@ lav_mvnorm_missing_scores_mu_vech_sigma <- function(Y           = NULL,
 
     out <- cbind(dmu, SC)
 
+    # weights
     if(!is.null(wt)) {
         out <- out * wt
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(P, el.idx = x.idx, meanstructure = TRUE)
+        out[, !not.x] <- 0
+    }
+
     out
 }
 
@@ -817,19 +936,23 @@ lav_mvnorm_missing_logl_hessian_data <- function(Y           = NULL,
                                                  wt          = NULL,
                                                  Mu          = NULL,
                                                  Sigma       = NULL,
+                                                 x.idx       = NULL,
                                                  Sinv.method = "eigen",
                                                  Sigma.inv   = NULL) {
     # missing patterns
     Yp <- lav_samplestats_missing_patterns(Y = Y, Mp = Mp, wt = wt)
 
     lav_mvnorm_missing_logl_hessian_samplestats(Yp = Yp, Mu = Mu,
-        Sigma = Sigma, Sinv.method = Sinv.method, Sigma.inv = Sigma.inv)
+        Sigma = Sigma, x.idx = x.idx, Sinv.method = Sinv.method,
+        Sigma.inv = Sigma.inv)
 }
 
 lav_mvnorm_missing_logl_hessian_samplestats <-
     function(Yp          = NULL,
+             # wt not needed
              Mu          = NULL,
              Sigma       = NULL,
+             x.idx       = NULL,
              Sinv.method = "eigen",
              Sigma.inv   = NULL) {
 
@@ -869,8 +992,8 @@ lav_mvnorm_missing_logl_hessian_samplestats <-
         tmp21[var.idx,1] <- sigma.inv %*% (Yp[[p]]$MY - Mu[var.idx])
 
         W.tilde <- Yp[[p]]$SY + tcrossprod(Yp[[p]]$MY - Mu[var.idx])
-        AAA <- ( sigma.inv %*% 
-                 (2*W.tilde - Sigma[var.idx,var.idx,drop = FALSE]) %*% 
+        AAA <- ( sigma.inv %*%
+                 (2*W.tilde - Sigma[var.idx,var.idx,drop = FALSE]) %*%
                  sigma.inv )
         tmp22 <- matrix(0, P, P)
         tmp22[var.idx, var.idx] <- AAA
@@ -886,14 +1009,24 @@ lav_mvnorm_missing_logl_hessian_samplestats <-
 
     H12 <- t(H21)
 
-    -1 * rbind( cbind(H11, H12),
-                cbind(H21, H22) )
+    out <- -1 * rbind( cbind(H11, H12),
+                       cbind(H21, H22) )
+
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(nvar = P, el.idx = x.idx,
+                                     meanstructure = TRUE)
+        out[, !not.x] <- 0
+        out[!not.x, ] <- 0
+    }
+
+    out
 }
 
 
 
 
-# 5) Information 
+# 5) Information
 
 # 5a: expected unit information Mu and vech(Sigma)
 #     (only useful under MCAR)
@@ -903,6 +1036,7 @@ lav_mvnorm_missing_information_expected <- function(Y           = NULL,
                                                     wt          = NULL,
                                                     Mu          = NULL,# unused
                                                     Sigma       = NULL,
+                                                    x.idx       = NULL,
                                                     Sigma.inv   = NULL,
                                                     Sinv.method = "eigen") {
 
@@ -920,7 +1054,11 @@ lav_mvnorm_missing_information_expected <- function(Y           = NULL,
 
     # N
     if(!is.null(wt)) {
-        N <- sum(wt)
+        if(length(Mp$empty.idx) > 0L) {
+            N <- sum(wt) - sum(wt[Mp$empty.idx])
+        } else {
+            N <- sum(wt)
+        }
     } else {
         N <- sum(Mp$freq) # removed empty cases!
     }
@@ -950,7 +1088,7 @@ lav_mvnorm_missing_information_expected <- function(Y           = NULL,
 
         S2.inv <- 0.5 * lav_matrix_duplication_pre_post(S.inv %x% S.inv)
 
-        if(!is.null(wt)) { 
+        if(!is.null(wt)) {
             FREQ <- sum( wt[ Mp$case.idx[[p]] ] )
         } else {
             FREQ <- Mp$freq[p]
@@ -960,7 +1098,17 @@ lav_mvnorm_missing_information_expected <- function(Y           = NULL,
         I22 <- I22 + FREQ * S2.inv
     }
 
-    lav_matrix_bdiag(I11, I22)/N
+    out <- lav_matrix_bdiag(I11, I22)/N
+
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(nvar = P, el.idx = x.idx,
+                                     meanstructure = TRUE)
+        out[!not.x, ] <- 0
+        out[, !not.x] <- 0
+    }
+
+    out
 }
 
 # 5b: unit observed information Mu and vech(Sigma) from raw data
@@ -970,6 +1118,7 @@ lav_mvnorm_missing_information_observed_data <- function(Y           = NULL,
                                                          wt          = NULL,
                                                          Mu          = NULL,
                                                          Sigma       = NULL,
+                                                         x.idx       = NULL,
                                                          Sinv.method = "eigen",
                                                          Sigma.inv   = NULL) {
     # missing patterns
@@ -979,34 +1128,40 @@ lav_mvnorm_missing_information_observed_data <- function(Y           = NULL,
 
     # N
     if(!is.null(wt)) {
-        N <- sum(wt)
+        if(length(Mp$empty.idx) > 0L) {
+            N <- sum(wt) - sum(wt[Mp$empty.idx])
+        } else {
+            N <- sum(wt)
+        }
     } else {
-        N <- sum(Mp$freq)
+        N <- sum(Mp$freq) # removed empty cases!
     }
 
     # observed information
     observed <- lav_mvnorm_missing_logl_hessian_data(Y = Y, Mp = Mp, wt = wt,
-                    Mu = Mu, Sigma = Sigma, Sinv.method = Sinv.method, 
-                    Sigma.inv = Sigma.inv)
+                    Mu = Mu, Sigma = Sigma, x.idx = x.idx,
+                    Sinv.method = Sinv.method, Sigma.inv = Sigma.inv)
 
     -observed/N
 }
 
 # 5b-bis: unit observed information Mu and vech(Sigma) from samplestats
 lav_mvnorm_missing_information_observed_samplestats <-
-    function(Yp          = NULL, 
+    function(Yp          = NULL,
+             # wt not needed
              Mu          = NULL,
              Sigma       = NULL,
+             x.idx       = NULL,
              Sinv.method = "eigen",
              Sigma.inv   = NULL) {
 
     N <- sum(sapply(Yp, "[[", "freq")) # implicitly: removed empty cases!
-    
+
     # observed information
     observed <- lav_mvnorm_missing_logl_hessian_samplestats(Yp = Yp, Mu = Mu,
-                    Sigma = Sigma, Sinv.method = Sinv.method,
+                    Sigma = Sigma, x.idx = x.idx, Sinv.method = Sinv.method,
                     Sigma.inv = Sigma.inv)
-    
+
     -observed/N
 }
 
@@ -1015,8 +1170,10 @@ lav_mvnorm_missing_information_observed_samplestats <-
 lav_mvnorm_missing_information_firstorder <- function(Y           = NULL,
                                                       Mp          = NULL,
                                                       wt          = NULL,
+                                                      cluster.idx = NULL,
                                                       Mu          = NULL,
                                                       Sigma       = NULL,
+                                                      x.idx       = NULL,
                                                       Sinv.method = "eigen",
                                                       Sigma.inv   = NULL) {
     # missing patterns
@@ -1026,15 +1183,30 @@ lav_mvnorm_missing_information_firstorder <- function(Y           = NULL,
 
     # N
     if(!is.null(wt)) {
-        N <- sum(wt)
+        if(length(Mp$empty.idx) > 0L) {
+            N <- sum(wt) - sum(wt[Mp$empty.idx])
+        } else {
+            N <- sum(wt)
+        }
     } else {
-        N <- sum(Mp$freq)
+        N <- sum(Mp$freq) # removed empty cases!
     }
 
-    SC <- lav_mvnorm_missing_scores_mu_vech_sigma(Y = Y, Mp = Mp, wt = wt, 
-              Mu = Mu, Sigma = Sigma, Sinv.method = Sinv.method, 
+    SC <- lav_mvnorm_missing_scores_mu_vech_sigma(Y = Y, Mp = Mp, wt = wt,
+              Mu = Mu, Sigma = Sigma, x.idx = x.idx, Sinv.method = Sinv.method,
               Sigma.inv = Sigma.inv)
 
+    # handle clustering
+    if(!is.null(cluster.idx)) {
+        # take the sum within each cluster
+        SC <- rowsum(SC, group = cluster.idx, reorder = FALSE, na.rm = TRUE)
+
+        # lower bias is number of clusters is not very high
+        nC <- nrow(SC)
+        correction.factor <- nC / (nC - 1)
+        SC <- SC * sqrt(correction.factor)
+    }
+
     lav_matrix_crossprod(SC)/N
 }
 
@@ -1043,8 +1215,10 @@ lav_mvnorm_missing_information_firstorder <- function(Y           = NULL,
 lav_mvnorm_missing_information_both <- function(Y           = NULL,
                                                 Mp          = NULL,
                                                 wt          = NULL,
+                                                cluster.idx = NULL,
                                                 Mu          = NULL,
                                                 Sigma       = NULL,
+                                                x.idx       = NULL,
                                                 Sinv.method = "eigen",
                                                 Sigma.inv   = NULL,
                                                 information = "observed") {
@@ -1055,7 +1229,7 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
         Sigma.inv <- lav_matrix_symmetric_inverse(S = Sigma, logdet = FALSE,
                                                   Sinv.method = Sinv.method)
     }
-    
+
     # for the tcrossprod
     idx1 <- lav_matrix_vech_col_idx(P); idx2 <- lav_matrix_vech_row_idx(P)
 
@@ -1073,9 +1247,13 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
 
     # N
     if(!is.null(wt)) {
-        N <- sum(wt)
+        if(length(Mp$empty.idx) > 0L) {
+            N <- sum(wt) - sum(wt[Mp$empty.idx])
+        } else {
+            N <- sum(wt)
+        }
     } else {
-        N <- sum(Mp$freq)
+        N <- sum(Mp$freq) # removed empty cases!
     }
 
     # subtract Mu
@@ -1135,21 +1313,21 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
             I22 <- I22 + FREQ * S2.inv
         } else {
             pat.freq <- Yp[[p]]$freq
-        
+
             tmp21 <- matrix(0,P,1)
             tmp21[var.idx,1] <- sigma.inv %*% (Yp[[p]]$MY - Mu[var.idx])
-        
+
             W.tilde <- Yp[[p]]$SY + tcrossprod(Yp[[p]]$MY - Mu[var.idx])
-            AAA <- ( sigma.inv %*%  
+            AAA <- ( sigma.inv %*%
                      (2*W.tilde - Sigma[var.idx,var.idx,drop = FALSE]) %*%
                      sigma.inv )
             tmp22 <- matrix(0, P, P)
             tmp22[var.idx, var.idx] <- AAA
-        
+
             i11 <- S.inv
             i21 <- lav_matrix_duplication_pre( tmp21 %x% S.inv )
             i22 <- (1/2) * lav_matrix_duplication_pre_post(S.inv %x% tmp22)
-        
+
             I11 <- I11 + pat.freq * i11
             I21 <- I21 + pat.freq * i21
             I22 <- I22 + pat.freq * i22
@@ -1175,10 +1353,28 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
     # add dmu
     SC <- cbind(dmu, SC)
 
+    # weights
     if(!is.null(wt)) {
         SC <- SC * wt
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(P, el.idx = x.idx, meanstructure = TRUE)
+        SC[, !not.x] <- 0
+    }
+
+    # handle clustering
+    if(!is.null(cluster.idx)) {
+        # take the sum within each cluster
+        SC <- rowsum(SC, group = cluster.idx, reorder = FALSE, na.rm = TRUE)
+
+        # lower bias is number of clusters is not very high
+        nC <- nrow(SC)
+        correction.factor <- nC / (nC - 1)
+        SC <- SC * sqrt(correction.factor)
+    }
+
     # first order information
     Bbeta <- lav_matrix_crossprod(SC)/N
 
@@ -1190,6 +1386,14 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
                          cbind(I21,   I22) )/N
     }
 
+    # fixed.x?
+    if(length(x.idx) > 0L) {
+        not.x <- eliminate.pstar.idx(nvar = P, el.idx = x.idx,
+                                     meanstructure = TRUE)
+        Abeta[!not.x, ] <- 0
+        Abeta[, !not.x] <- 0
+    }
+
     list(Abeta = Abeta, Bbeta = Bbeta)
 }
 
@@ -1253,6 +1457,7 @@ lav_mvnorm_missing_information_both <- function(Y           = NULL,
 # 10) other stuff
 
 # single imputation missing cells, under the normal model, pattern-based
+# FIXME: add wt
 lav_mvnorm_missing_impute_pattern <- function(Y           = NULL,
                                               Mp          = NULL,
                                               Mu          = NULL,
@@ -1360,7 +1565,7 @@ lav_mvnorm_missing_estep <- function(Y           = NULL,
 
             if(!is.null(wt)) {
                 WT <- wt[Mp$case.idx[[p]]]
-                T1 <- T1 + colSums(WT * O) 
+                T1 <- T1 + colSums(WT * O)
                 T2 <- T2 + crossprod(sqrt(WT) * O)
             } else {
                 # complete pattern
@@ -1413,7 +1618,7 @@ lav_mvnorm_missing_estep <- function(Y           = NULL,
             T2.pat[!var.idx, !var.idx] <-
                 T2.pat[!var.idx, !var.idx] + (T2.p11 * sum(WT))
         } else {
-            T2.pat[!var.idx, !var.idx] <- 
+            T2.pat[!var.idx, !var.idx] <-
                 T2.pat[!var.idx, !var.idx] + (T2.p11 * Mp$freq[[p]])
         }
 
