@@ -25,6 +25,30 @@ lav_partable_unrestricted <- function(lavobject      = NULL,
 
 }
 
+# generate parameter table for an independence model
+# YR - 12 Sep 2017: special case of lav_partable_unrestricted()
+lav_partable_independence <- function(lavobject      = NULL,
+                                      # if no object is available,
+                                      lavdata        = NULL,
+                                      lavpta         = NULL,
+                                      lavoptions     = NULL,
+                                      lavsamplestats = NULL,
+                                      # optional user-provided sample stats
+                                      sample.cov     = NULL,
+                                      sample.mean    = NULL,
+                                      sample.slopes  = NULL,
+                                      sample.th      = NULL,
+                                      sample.th.idx  = NULL,
+                                      sample.cov.x   = NULL,
+                                      sample.mean.x  = NULL) {
+
+    lav_partable_indep_or_unrestricted(lavobject = lavobject,
+        lavdata = lavdata, lavpta = lavpta, lavoptions = lavoptions,
+        lavsamplestats = lavsamplestats, sample.cov = sample.cov,
+        sample.mean = sample.mean , sample.slopes = sample.slopes,
+        sample.th = sample.th, sample.th.idx = sample.th.idx,
+        independent = TRUE)
+}
 
 lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                                       # if no object is available,
@@ -157,7 +181,7 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
         sample.mean.x <- SAMPLE.mean.x[[g]]
 
         # force local sample.cov to be pd -- just for starting values anyway
-        if(!is.null(sample.cov)) {
+        if(!is.null(sample.cov) && !anyNA(sample.cov)) {
             sample.cov <- lav_matrix_symmetric_force_pd(sample.cov)
         }
 
@@ -391,13 +415,13 @@ lav_partable_indep_or_unrestricted <- function(lavobject      = NULL,
                     # fix variances/covariances
                     exo.idx <- which(rhs %in% ov.names.x &
                                      lhs %in% ov.names.x &
-                                     op == "~~" & group == g)
+                                     op == "~~" & group == g) # ok
                     exo[exo.idx] <- 1L
                     free[exo.idx] <- 0L
 
                     # fix means
                     exo.idx <- which(lhs %in% ov.names.x &
-                                     op == "~1" & group == g)
+                                     op == "~1" & group == g) # ok
                     exo[exo.idx] <- 1L
                     free[exo.idx] <- 0L
                 }
