@@ -65,6 +65,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                    "lv.nonnormal",# latent variables with non-normal indicators
                    "lv.interaction", # interaction terms
                    "lv.efa",      # latent variables involved in efa
+                   "lv.rv",       # random slopes, random variables
                    "lv.ind",      # latent indicators (higher-order cfa)
                    "lv.marker",   # marker indicator per lv
 
@@ -73,9 +74,11 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
                   )
 
     # sanity check
-    stopifnot(is.list(partable),
-              !missing(type),
-              type %in% c(type.list, "all"))
+    stopifnot(is.list(partable), !missing(type))
+
+    if(!type %in% c(type.list, "all")) {
+        stop("lavaan ERROR: type =", dQuote(type), " is not a valid option")
+    }
 
     if(length(type) == 1L && type == "all") {
         type <- type.list
@@ -136,6 +139,13 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         }
     }
 
+    # random slope names, if any (new in 0.6-7)
+    if(!is.null(partable$rv) && any(nchar(partable$rv) > 0L)) {
+        RV.names <- unique(partable$rv[nchar(partable$rv) > 0L])
+    } else {
+        RV.names <- character(0L)
+    }
+
 
 
     # output: list per block
@@ -164,6 +174,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
     OUT$lv.nonnormal   <- vector("list", length = nblocks)
     OUT$lv.interaction <- vector("list", length = nblocks)
     OUT$lv.efa         <- vector("list", length = nblocks)
+    OUT$lv.rv          <- vector("list", length = nblocks)
     OUT$lv.ind         <- vector("list", length = nblocks)
     OUT$lv.marker      <- vector("list", length = nblocks)
 
@@ -176,6 +187,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         lv.names <- unique( partable$lhs[ partable$block == b  &
                                           (partable$op == "=~" |
                                            partable$op == "<~")  ] )
+        # including random slope names
+        lv.names2 <- unique(c(lv.names, RV.names))
 
         # determine lv interactions
         int.names <- unique(partable$rhs[ partable$block == b  &
@@ -198,6 +211,7 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             }
             lv.interaction <- int.names[ok.idx]
             lv.names <- c(lv.names, lv.interaction)
+            lv.names2 <- c(lv.names2, lv.interaction)
         } else {
             lv.interaction <- character(0L)
         }
@@ -210,7 +224,8 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         # regular latent variables ONLY (ie defined by =~ only)
         if("lv.regular" %in% type) {
             out <- unique( partable$lhs[ partable$block == b &
-                                         partable$op == "=~"   ] )
+                                         partable$op == "=~" &
+                                         !partable$lhs %in% RV.names ] )
             OUT$lv.regular[[b]] <- out
         }
 
@@ -240,6 +255,18 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             OUT$lv.efa[[b]] <- out
         }
 
+        # lv's that are random slopes
+        if("lv.rv" %in% type) {
+            if(is.null(partable$rv)) {
+                out <- character(0L)
+            } else {
+                out <- unique( partable$lhs[ partable$op == "=~" &
+                                             partable$block == b &
+                                             partable$lhs %in% RV.names ] )
+            }
+            OUT$lv.rv[[b]] <- out
+        }
+
         # lv's that are indicators of a higher-order factor
         if("lv.ind" %in% type) {
             out <- unique( partable$rhs[ partable$block == b &
@@ -284,11 +311,11 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
         if(!(length(type) == 1L &&
              type %in% c("lv", "lv.regular", "lv.nonnormal", "lv.x","lv.y"))) {
             # 1. indicators, which are not latent variables themselves
-            ov.ind <- v.ind[ !v.ind %in% lv.names ]
+            ov.ind <- v.ind[ !v.ind %in% lv.names2 ]
             # 2. dependent ov's
-            ov.y <- eqs.y[ !eqs.y %in% c(lv.names, ov.ind) ]
+            ov.y <- eqs.y[ !eqs.y %in% c(lv.names2, ov.ind) ]
             # 3. independent ov's
-            ov.x <- eqs.x[ !eqs.x %in% c(lv.names, ov.ind, ov.y) ]
+            ov.x <- eqs.x[ !eqs.x %in% c(lv.names2, ov.ind, ov.y) ]
         }
 
         # observed variables
@@ -301,15 +328,15 @@ lav_partable_vnames <- function(partable, type = NULL, ...,
             # 4. orphaned covariances
             ov.cov <- c(partable$lhs[ partable$block == b &
                                       partable$op == "~~" &
-                                     !partable$lhs %in% lv.names ],
+                                     !partable$lhs %in% lv.names2 ],
                         partable$rhs[ partable$block == b &
                                       partable$op == "~~" &
-                                     !partable$rhs %in% lv.names ])
+                                     !partable$rhs %in% lv.names2 ])
             # 5. orphaned intercepts/thresholds
             ov.int <- partable$lhs[ partable$block == b &
                                     (partable$op == "~1" |
                                      partable$op == "|") &
-                                    !partable$lhs %in% lv.names ]
+                                    !partable$lhs %in% lv.names2 ]
 
             ov.tmp <- c(ov.ind, ov.y, ov.x)
             ov.extra <- unique(c(ov.cov, ov.int)) # must be in this order!
